@@ -34,9 +34,7 @@
 #define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #endif
 
-#define KERNEL_MIN_VER(a,b,c) (LINUX_VERSION_CODE >= KERNEL_VERSION(a,b,c))
-
-#if !KERNEL_MIN_VER(2,1,115)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,115)
 /*
  * It might work with smaller kernels, but I never tested that.
  */
@@ -144,6 +142,11 @@
 #define PCI_DEVICE_ID_3DFX_VOODOO4 9
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,73)
+#define pci_get_device pci_find_device
+#define pci_dev_put(dev)
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 static char name_3dfx[] = "3dfx";
 
@@ -162,6 +165,7 @@ static struct pci_device_id id_table_3dfx[] = {
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
 	{PCI_VENDOR_ID_3DFX,		PCI_DEVICE_ID_3DFX_VOODOO4,
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
+	{ 0, }
 };
 #else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) */
 static struct pci_card {
@@ -188,7 +192,7 @@ static struct pci_card {
  * This macro is for accessing vma->vm_offset or vma->vm_pgoff depending
  * on kernel version.
  */
-#if !KERNEL_MIN_VER(2,3,14)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,14)
 #define VM_OFFSET(vma) (vma->vm_offset)
 #define VM_OFFSET_ALIGNED(vma) ((vma->vm_offset) & ~PAGE_MASK)
 #else
@@ -196,10 +200,10 @@ static struct pci_card {
 #define VM_OFFSET_ALIGNED(vma) ((vma->vm_pgoff) & ~PAGE_MASK)
 #endif
 
-#if KERNEL_MIN_VER(2,6,10)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
 #define my_remap_page_range(vma, start, ofs, len, prot) \
 	remap_pfn_range(vma, start, ofs >> PAGE_SHIFT, len, prot)
-#elif KERNEL_MIN_VER(2,6,0)
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 #define my_remap_page_range(vma, start, ofs, len, prot) \
 	remap_page_range(vma, start, ofs, len, prot)
 #else
@@ -241,8 +245,9 @@ void cleanup_module(void);
 static cardInfo cards[MAXCARDS];
 static int numCards = 0;
 #if HAVE_DEVFS
-#if KERNEL_MIN_VER(2,3,46) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) \
-    || defined(DEVFS_SUPPORT)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,46) && \
+    LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) || \
+    defined(DEVFS_SUPPORT)
 static devfs_handle_t devfs_handle;
 #endif
 #endif
@@ -252,7 +257,7 @@ static void findCardType(int vendor, int device)
 {
 	struct pci_dev *dev = NULL;
 
-	while (numCards < MAXCARDS && (dev = pci_find_device(vendor, device, dev))) {
+	while (numCards < MAXCARDS && (dev = pci_get_device(vendor, device, dev))) {
 		pci_read_config_dword(dev, PCI_BASE_ADDRESS_0, &cards[numCards].addr0);
 		pci_read_config_dword(dev, PCI_BASE_ADDRESS_1, &cards[numCards].addr1);
 		pci_read_config_dword(dev, PCI_BASE_ADDRESS_2, &cards[numCards].addr2);
@@ -656,8 +661,8 @@ static int setmtrr_3dfx(void)
 	 * First do a bios fixup if this system has a 82441FX chipset.
 	 */
 	struct pci_dev *dev = NULL;
-	dev = pci_find_device(PCI_VENDOR_ID_INTEL,
-			      PCI_DEVICE_ID_INTEL_82371SB_0, dev);
+	dev = pci_get_device(PCI_VENDOR_ID_INTEL,
+	                     PCI_DEVICE_ID_INTEL_82371SB_0, dev);
 	if (dev) {
 		pci_read_config_byte(dev, 0x82, &dlc);
 		if (!(dlc & 1 << 1)) {
@@ -665,6 +670,7 @@ static int setmtrr_3dfx(void)
 			pci_write_config_byte(dev, 0x82, dlc);
 			printk("3dfx: PIIX3: Enabling Passive Release\n");
 		}
+		pci_dev_put(dev);
 	}
 
 	/*
@@ -763,7 +769,7 @@ static int resetmtrr_3dfx(void)
 #endif /* CONFIG_MTRR */
 
 static struct file_operations fops_3dfx = {
-#if KERNEL_MIN_VER(2,4,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 	.owner =	THIS_MODULE,
 #endif
 	.ioctl =	ioctl_3dfx,
@@ -822,11 +828,11 @@ int init_module(void)
 	}
 
 #if HAVE_DEVFS
-#if KERNEL_MIN_VER(2,6,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	devfs_mk_cdev(MKDEV(MAJOR_3DFX, DEVICE_VOODOO),
 		      S_IFCHR | S_IROTH | S_IWOTH | S_IRGRP | S_IWGRP,
 		      "3dfx");
-#elif KERNEL_MIN_VER(2,3,46) || defined(DEVFS_SUPPORT)
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,46) || defined(DEVFS_SUPPORT)
 	devfs_handle = devfs_register(NULL, "3dfx", DEVFS_FL_NONE,
 		MAJOR_3DFX, DEVICE_VOODOO,
 		S_IFCHR | S_IROTH | S_IWOTH | S_IRGRP | S_IWGRP,
@@ -863,17 +869,14 @@ void cleanup_module(void)
 	resetmtrr_3dfx();
 #endif
 #if HAVE_DEVFS
-#if KERNEL_MIN_VER(2,6,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	devfs_remove("3dfx");
-#elif KERNEL_MIN_VER(2,3,46) || defined(DEVFS_SUPPORT)
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,46) || defined(DEVFS_SUPPORT)
 	devfs_unregister(devfs_handle);
 #endif
 #endif
 
-	if (unregister_chrdev(MAJOR_3DFX, "3dfx")) {
-		DEBUGMSG(("3dfx: unregister_chrdev failed\n"));
-		return;
-	}
+	unregister_chrdev(MAJOR_3DFX, "3dfx");
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	pci_unregister_driver(&driver_3dfx);
@@ -900,12 +903,12 @@ long init_3dfx(long mem_start, long mem_end)
 }
 #endif /* !MODULE */
 
-#if KERNEL_MIN_VER(2,1,21)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,21)
 MODULE_AUTHOR("Daryll Strauss et al.");
 MODULE_DESCRIPTION("3dfx glide kernel device driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
-MODULE_VERSION("2007.02.06");
+MODULE_VERSION("2008.06.24");
 #endif
 #endif
 
